#ifndef OBJECT_H
#define OBJECT_H

#define MAX_TOK_CNT 128 // 128
#define MAX_ENV_ELM 50  // 50

#define FOR_POINTER_IN_LIST(_list) \
    if(_list && _list->type == TYPE_LIST) \
    for(Object *_element = _list->list; \
            _element != NULL;\
            _element = _element->forward)
#define POINTER _element

#define FOR_POINTERS_IN_LISTS(_list, _list2) \
    for(Object *_element = _list->list, *_element2 = _list2->list; \
            _element != NULL && _element2 != NULL; \
            _element = _element->forward, _element2 = _element2->forward)
#define P1 POINTER
#define P2 _element2

enum errorCode {
    MISMATCHED_PARENS,
    BAD_LIST_OF_SYMBOL_STRINGS,
    TYPE_LIST_NOT_CAUGHT,
    NULL_ENV,
    EMPTY_ENV,
    BUILT_IN_NOT_FOUND,
    NULL_PARSE,
    NULL_LAMBDA_LIST,
    NULL_MAP_ARGS,
    LAMBDA_ARGS_NOT_LIST,
    DID_NOT_FIND_SYMBOL,
    BAD_TYPE,
    UNEXPECTED_FORM,
    LISTS_NOT_SAME_SIZE,
    BAD_NUMBER,
    UNSUPPORTED_NUMBER_TYPE,
    NOT_A_SYMBOL,
    ONLY_ONE_ARGUMENT,
    NOT_A_LIST,
    SCRIPT_NOT_FOUND,
    NO_CLONE_SPECIFIED
};

#define MALLOC_FLAG 64

typedef enum Type {
    TYPE_NUMBER,
    TYPE_BOOL,
    TYPE_LIST,
    TYPE_FUNC,
    TYPE_SYMBOL,
    TYPE_LAMBDA,
    TYPE_STRING,
    TYPE_OTHER,
    TYPE_ERROR
} Type;

typedef struct Object Object;
struct Lambda;
struct Environment;
struct Slice;
struct Other;

struct Object {
    Type type;
    Object *forward;
    union {
        int number;
        Object *list;
        char *string;
        Object (*func)(Object, Object, struct Environment *);
        struct Lambda *lambda;
        struct Other *other;
        enum errorCode err;
    };
};

struct Lambda {
    Object params;
    Object body;
};

struct Other {
    void (*cleanup)(Object*);
    Object (*clone)(struct Other*);
    void *data;
};

char* stringObj(char *dest, const Object *obj);
void printList(const Object *list);
void printObj(const Object *obj);
void debugObj(const Object *obj);
void printErr(const Object *obj);

int isEmpty(const Object *obj);
Object *tail(const Object *listObj);
void insertIntoList(Object *dest, int ind, const Object src);
void replaceListing(Object *list, int i, const Object src);
void nf_addToList(Object *dest, Object src);
void deleteList(Object *dest);

int listLength(const Object *listObj);
Object *itemAt(const Object *listObj, int n);
void copyList(Object *dest, const Object *src);

void cleanObject(Object *target);
void printAndClean(Object *target);
void allocObject(Object **spot, const Object src);
void appendList(Object *dest, const Object *src);

int isStringy(const Object test);
int isValidType(const Object test);
int isError(const Object obj, const enum errorCode err);

Object cloneList(const Object src);
Object cloneString(Object obj);
Object cloneLambda(const Object old);
Object cloneObject(const Object src);

Object newObject(Type type);
Object listObject();
Object startList(const Object start);
Object objFromSlice(const char *string, int len);
Object symFromSlice(const char *string, int len);
Object boolObject(int b);
Object numberObject(int num);
Object otherObject();
Object errorObject(enum errorCode err);
Object constructLambda(const Object *params, const Object *body);

// Object version of listLength()
Object len(Object obj1, Object, struct Environment *);

#endif
